#include "stdint.h"
#include "s3c44b0x.h"
#include "delay.h"
#include "bitbang.h"
#include "uart.h"
#include "nand.h"
#define BFF 0xFFFFFFFF
void nand_erase(uint32_t sector);
void nand_compare(uint32_t sector, uint32_t *buffer);
void nand_write(uint32_t sector, uint32_t* buffer);

void cpu_init()
{
   /* Set clock rate to 66.428MHz, just in case the JB has
      it set wrong
   */
   *PLLCON=0x55051;

   /* Be sure we're in cacheable memory
      Can the framebuffer be in cache too?
   */
   *NCACHBE0=0x80002000;
   *NCACHBE1=0x0;
}

void set_lcd_brightness(uint32_t brightness)
{
   /* Set LCD backlight brightness
      0 is dark, 0xa00 is full bright
   */
   *TCMPB1=brightness;
}


void write_memory (void)
{
 uint32_t i;
 int32_t len;
 uint32_t* myadd;
 myadd= (uint32_t*)0x0c005000;
 len=60000;
/* for(i=0;i<len;i++) 
   *(myadd+i)=0; */
 i=0;
 while(len > 0) {
  if((i%32)== 0)
     nand_erase(i);
   nand_write(i,myadd+ (i*128));
   nand_compare(i,myadd+(i*128));
   len=len-128;
    uart_tx_string((const uint8_t*)"\r\n   length is  0x");
     uart_tx_string((const uint8_t*)itoh(len));  
   i++;
}
}


uint32_t compare_fail=0;

/* Compare a MMC sector to a memory buffer */
void mmc_compare(uint32_t sector, uint8_t* start)
{
   uint32_t i;

   mmc_byte(0xff);
   mmc_byte(0xff);
   mmc_byte(0xff);
   mmc_byte(0xff);

   /* read sector command */
   mmc_command(17, sector);

   /* spin until we get a zero reply, indicating command accepted */
   do
   {
      i=mmc_reply(1);
   } while (i!=0);


   /* spin until start of data token is received (0xfe for single read) */
   do
   {
      i=mmc_reply(1);
   } while (i!=0xfe);

   /* Read from the card */
   for (i=0; i<512; i++)
   {
      /* Total the bytes that diff */
      if (mmc_reply(1)!=(*start))
      {
         compare_fail++;
      }
      ++start;
   }

   /* give the MMC state machine 8 extra clocks */
   mmc_byte(0xff);
}

uint32_t temp_buffer[256];    /* 1k of ram */

/* Copy and readback to the MMC */
void copy_test()
{
   uint32_t not_erased;
   compare_fail=0;
   not_erased=0;
   nand_test(); 
nand_erase(0x0000000);
//nand_erase(0x1);
 write_memory();

  
   uart_tx_string((const uint8_t*)"\r\n over and out.");
}




int main(void)
{
   int32_t i;

   /* Init */
   cpu_init();
   uart_init();

   /* cool fade out to indicate demo running */
   for (i=0xa00; i>=0; i-=4)
   {
      set_lcd_brightness(i);
      delay_ms(1);
   }

   uart_tx_string((const uint8_t*)"\r\n----------------------------------------------");
   uart_tx_string((const uint8_t*)"\r\nFile: " __FILE__ ", (" __DATE__ ", " __TIME__);
   uart_tx_string((const uint8_t*)")\r\n");
   copy_test();
   uart_tx_string((const uint8_t*)"\r\n\r test complete..");
   uart_tx_string((const uint8_t*)"\r\n----------------------------------------------\r\n");

   /* Fade back in to indicate dump complete */
   for (i=0; i<=0xa00; i+=4)
   {
      set_lcd_brightness(i);
      delay_ms(1);
   }

   /* Spin forever */
   for (;;);

   return 0;
}
