#ifndef __S3C44B0X_H
#define __S3C44B0X_H

/* CPU register definitions for the Samsung S3C44B0X
   Assumes little endian mode, with all but the UART TX/RX
   holding registers as 32 bit words.  The RTC regs are probably
   supposed to be 8 bit, but they seem to work as 32.
   Read-only registers are not defined as such.
*/


/****** CPU WRAPPER */
#define SYSCFG ((uint32_t volatile* const) 0x01c00000) /* SYSCFG (System Configuration) */
#define NCACHBE0 ((uint32_t volatile* const) 0x01c00004) /* NCACHBE0 (Non Cacheable Area 0) */
#define NCACHBE1 ((uint32_t volatile* const) 0x01c00008) /* NCACHBE1 (Non Cacheable Area 1) */
#define SBUSCON ((uint32_t volatile* const) 0x01c40000) /* SBUSCON (System Bus Control) */

/****** MEMORY CONTROLLER */
#define BWSCON ((uint32_t volatile* const) 0x01c80000) /* BWSCON (Bus Width & Wait Status Control) */
#define BANKCON0 ((uint32_t volatile* const) 0x01c80004) /* BANKCON0 (Boot ROM Control) */
#define BANKCON1 ((uint32_t volatile* const) 0x01c80008) /* BANKCON1 (BANK1 Control) */
#define BANKCON2 ((uint32_t volatile* const) 0x01c8000c) /* BANKCON2 (BANK2 Control) */
#define BANKCON3 ((uint32_t volatile* const) 0x01c80010) /* BANKCON3 (BANK3 Control) */
#define BANKCON4 ((uint32_t volatile* const) 0x01c80014) /* BANKCON4 (BANK4 Control) */
#define BANKCON5 ((uint32_t volatile* const) 0x01c80018) /* BANKCON5 (BANK5 Control) */
#define BANKCON6 ((uint32_t volatile* const) 0x01c8001c) /* BANKCON6 (BANK6 Control) */
#define BANKCON7 ((uint32_t volatile* const) 0x01c80020) /* BANKCON7 (BANK7 Control) */
#define REFRESH ((uint32_t volatile* const) 0x01c80024) /* REFRESH (DRAM/SDRAM Refresh Control) */
#define BANKSIZE ((uint32_t volatile* const) 0x01c80028) /* BANKSIZE (Flexible Bank Size) */
#define MRSRB6 ((uint32_t volatile* const) 0x01c8002c) /* MRSRB6 (Mode register set for SDRAM) */
#define MRSRB7 ((uint32_t volatile* const) 0x01c80030) /* MRSRB7 (Mode register set for SDRAM) */

/****** UART */
#define ULCON0 ((uint32_t volatile* const) 0x01d00000) /* ULCON0 (UART 0 Line Control) */
#define ULCON1 ((uint32_t volatile* const) 0x01d04000) /* ULCON1 (UART 1 Line Control) */
#define UCON0 ((uint32_t volatile* const) 0x01d00004) /* UCON0 (UART 0 Control) */
#define UCON1 ((uint32_t volatile* const) 0x01d04004) /* UCON1 (UART 1 Control) */
#define UFCON0 ((uint32_t volatile* const) 0x01d00008) /* UFCON0 (UART 0 FIFO Control) */
#define UFCON1 ((uint32_t volatile* const) 0x01d04008) /* UFCON1 (UART 1 FIFO Control) */
#define UMCON0 ((uint32_t volatile* const) 0x01d0000c) /* UMCON0 (UART 0 Modem Control) */
#define UMCON1 ((uint32_t volatile* const) 0x01d0400c) /* UMCON1 (UART 1 Modem Control) */
#define UTRSTAT0 ((uint32_t volatile* const) 0x01d00010) /* UTRSTAT0 (UART 0 Tx/Rx Status) */
#define UTRSTAT1 ((uint32_t volatile* const) 0x01d04010) /* UTRSTAT1 (UART 1 Tx/Rx Status) */
#define UERSTAT0 ((uint32_t volatile* const) 0x01d00014) /* UERSTAT0 (UART 0 Rx Error Status) */
#define UERSTAT1 ((uint32_t volatile* const) 0x01d04014) /* UERSTAT1 (UART 1 Rx Error Status) */
#define UFSTAT0 ((uint32_t volatile* const) 0x01d00018) /* UFSTAT0 (UART 0 FIFO Status) */
#define UFSTAT1 ((uint32_t volatile* const) 0x01d04018) /* UFSTAT1 (UART 1 FIFO Status) */
#define UMSTAT0 ((uint32_t volatile* const) 0x01d0001c) /* UMSTAT0 (UART 0 Modem Status) */
#define UMSTAT1 ((uint32_t volatile* const) 0x01d0401c) /* UMSTAT1 (UART 1 Modem Status) */
#define UTXH0 ((uint8_t volatile* const) 0x01d00020) /* UTXH0 (UART 0 Transmission Hold) */
#define UTXH1 ((uint8_t volatile* const) 0x01d04020) /* UTXH1 (UART 1 Transmission Hold) */
#define URXH0 ((uint8_t volatile* const) 0x01d00024) /* URXH0 (UART 0 Receive Buffer) */
#define URXH1 ((uint8_t volatile* const) 0x01d04024) /* URXH1 (UART 1 Receive Buffer) */
#define UBRDIV0 ((uint32_t volatile* const) 0x01d00028) /* UBRDIV0 (UART 0 Baud Rate Divisor) */
#define UBRDIV1 ((uint32_t volatile* const) 0x01d04028) /* UBRDIV1 (UART 1 Baud Rate Divisor) */

/****** SIO */
#define SIOCON ((uint32_t volatile* const) 0x01d14000) /* SIOCON (SIO Control) */
#define SIODAT ((uint32_t volatile* const) 0x01d14004) /* SIODAT (SIO Data) */
#define SBRDR ((uint32_t volatile* const) 0x01d14008) /* SBRDR (SIO Baud Rate Prescaler) */
#define ITVCNT ((uint32_t volatile* const) 0x01d1400c) /* ITVCNT (SIO Interval Counter) */
#define DCNTZ ((uint32_t volatile* const) 0x01d14010) /* DCNTZ (SIO DMA Count Zero) */

/****** IIS */
#define IISCON ((uint32_t volatile* const) 0x01d18000) /* IISCON (IIS Control) */
#define IISMOD ((uint32_t volatile* const) 0x01d18004) /* IISMOD (IIS Mode) */
#define IISPSR ((uint32_t volatile* const) 0x01d18008) /* IISPSR (IIS Prescaler) */
#define IISFIFCON ((uint32_t volatile* const) 0x01d1800c) /* IISFIFCON (IIS FIFO Control) */
#define IISFIF ((uint32_t volatile* const) 0x01d18010) /* IISFIF (IIS FIFO Entry) */

/****** I/O PORT */
#define PCONA ((uint32_t volatile* const) 0x01d20000) /* PCONA (Port A Control) */
#define PDATA ((uint32_t volatile* const) 0x01d20004) /* PDATA (Port A Data) */
#define PCONB ((uint32_t volatile* const) 0x01d20008) /* PCONB (Port B Control) */
#define PDATB ((uint32_t volatile* const) 0x01d2000c) /* PDATB (Port B Data) */
#define PCONC ((uint32_t volatile* const) 0x01d20010) /* PCONC (Port C Control) */
#define PDATC ((uint32_t volatile* const) 0x01d20014) /* PDATC (Port C Data) */
#define PUPC ((uint32_t volatile* const) 0x01d20018) /* PUPC (Pull-up Control C) */
#define PCOND ((uint32_t volatile* const) 0x01d2001c) /* PCOND (Port D Control) */
#define PDATD ((uint32_t volatile* const) 0x01d20020) /* PDATD (Port D Data) */
#define PUPD ((uint32_t volatile* const) 0x01d20024) /* PUPD (Pull-up Control D) */
#define PCONE ((uint32_t volatile* const) 0x01d20028) /* PCONE (Port E Control) */
#define PDATE ((uint32_t volatile* const) 0x01d2002c) /* PDATE (Port E Data) */
#define PUPE ((uint32_t volatile* const) 0x01d20030) /* PUPE (Pull-up Control E) */
#define PCONF ((uint32_t volatile* const) 0x01d20034) /* PCONF (Port F Control) */
#define PDATF ((uint32_t volatile* const) 0x01d20038) /* PDATF (Port F Data) */
#define PUPF ((uint32_t volatile* const) 0x01d2003c) /* PUPF (Pull-up Control F) */
#define PCONG ((uint32_t volatile* const) 0x01d20040) /* PCONG (Port G Control) */
#define PDATG ((uint32_t volatile* const) 0x01d20044) /* PDATG (Port G Data) */
#define PUPG ((uint32_t volatile* const) 0x01d20048) /* PUPG (Pull-up Control G) */
#define SPUCR ((uint32_t volatile* const) 0x01d2004c) /* SPUCR (Special Pull-up) */
#define EXTINT ((uint32_t volatile* const) 0x01d20050) /* EXTINT (External Interrupt Control) */
#define EXTINPND ((uint32_t volatile* const) 0x01d20054) /* EXTINPND (External Interrupt Pending) */

/****** WATCHDOG TIMER */
#define WTCON ((uint32_t volatile* const) 0x01d30000) /* WTCON (Watchdog Timer Mode) */
#define WTDAT ((uint32_t volatile* const) 0x01d30004) /* WTDAT (Watchdog Timer Data) */
#define WTCNT ((uint32_t volatile* const) 0x01d30008) /* WTCNT (Watchdog Timer Count) */

/****** A/D CONVERTER */
#define ADCCON ((uint32_t volatile* const) 0x01d40000) /* ADCCON (ADC Control) */
#define ADCPSR ((uint32_t volatile* const) 0x01d40004) /* ADCPSR (ADC Prescaler) */
#define ADCDAT ((uint32_t volatile* const) 0x01d40008) /* ADCDAT (Digitized 10 bit Data) */

/****** PWM TIMER */
#define TCFG0 ((uint32_t volatile* const) 0x01d50000) /* TCFG0 (Timer Configuration) */
#define TCFG1 ((uint32_t volatile* const) 0x01d50004) /* TCFG1 (Timer Configuration) */
#define TCON ((uint32_t volatile* const) 0x01d50008) /* TCON (Timer Control) */
#define TCNTB0 ((uint32_t volatile* const) 0x01d5000c) /* TCNTB0 (Timer Count Buffer 0) */
#define TCMPB0 ((uint32_t volatile* const) 0x01d50010) /* TCMPB0 (Timer Compare Buffer 0) */
#define TCNTO0 ((uint32_t volatile* const) 0x01d50014) /* TCNTO0 (Timer Count Observation 0) */
#define TCNTB1 ((uint32_t volatile* const) 0x01d50018) /* TCNTB1 (Timer Count Buffer 1) */
#define TCMPB1 ((uint32_t volatile* const) 0x01d5001c) /* TCMPB1 (Timer Compare Buffer 1) */
#define TCNTO1 ((uint32_t volatile* const) 0x01d50020) /* TCNTO1 (Timer Count Observation 1) */
#define TCNTB2 ((uint32_t volatile* const) 0x01d50024) /* TCNTB2 (Timer Count Buffer 2) */
#define TCMPB2 ((uint32_t volatile* const) 0x01d50028) /* TCMPB2 (Timer Compare Buffer 2) */
#define TCNTO2 ((uint32_t volatile* const) 0x01d5002c) /* TCNTO2 (Timer Count Observation 2) */
#define TCNTB3 ((uint32_t volatile* const) 0x01d50030) /* TCNTB3 (Timer Count Buffer 3) */
#define TCMPB3 ((uint32_t volatile* const) 0x01d50034) /* TCMPB3 (Timer Compare Buffer 3) */
#define TCNTO3 ((uint32_t volatile* const) 0x01d50038) /* TCNTO3 (Timer Count Observation 3) */
#define TCNTB4 ((uint32_t volatile* const) 0x01d5003c) /* TCNTB4 (Timer Count Buffer 4) */
#define TCMPB4 ((uint32_t volatile* const) 0x01d50040) /* TCMPB4 (Timer Compare Buffer 4) */
#define TCNTO4 ((uint32_t volatile* const) 0x01d50044) /* TCNTO4 (Timer Count Observation 4) */
#define TCNTB5 ((uint32_t volatile* const) 0x01d50048) /* TCNTB5 (Timer Count Buffer 5) */
#define TCNTO5 ((uint32_t volatile* const) 0x01d5004c) /* TCNTO5 (Timer Count Observation 5) */

/****** IIC */
#define IICCON ((uint32_t volatile* const) 0x01d60000) /* IICCON (IIC Control) */
#define IICSTAT ((uint32_t volatile* const) 0x01d60004) /* IICSTAT (IIC Status) */
#define IICADD ((uint32_t volatile* const) 0x01d60008) /* IICADD (IIC Address) */
#define IICDS ((uint32_t volatile* const) 0x01d6000c) /* IICDS (IIC Data Shift) */

/****** RTC */
/* THE BOOK SHOWS THESe as 8 bit registers, not 32.  Double-check? */
#define RTCCON ((uint32_t volatile* const) 0x01d70040) /* RTCCON (RTC Control) */
#define RTCALM ((uint32_t volatile* const) 0x01d70050) /* RTCALM (RTC Alarm) */
#define ALMSEC ((uint32_t volatile* const) 0x01d70054) /* ALMSEC (Alarm Second) */
#define ALMMIN ((uint32_t volatile* const) 0x01d70058) /* ALMMIN (Alarm Minute) */
#define ALMHOUR ((uint32_t volatile* const) 0x01d7005c) /* ALMHOUR (Alarm Hour) */
#define ALMDAY ((uint32_t volatile* const) 0x01d70060) /* ALMDAY (Alarm Day) */
#define ALMMON ((uint32_t volatile* const) 0x01d70064) /* ALMMON (Alarm Month) */
#define ALMYEAR ((uint32_t volatile* const) 0x01d70068) /* ALMYEAR (Alarm Year) */
#define RTCRST ((uint32_t volatile* const) 0x01d7006c) /* RTCRST (RTC Round Reset) */
#define BCDSEC ((uint32_t volatile* const) 0x01d70070) /* BCDSEC (BCD Second) */
#define BCDMIN ((uint32_t volatile* const) 0x01d70074) /* BCDMIN (BCD Minute) */
#define BCDHOUR ((uint32_t volatile* const) 0x01d70078) /* BCDHOUR (BCD Hour) */
#define BCDDAY ((uint32_t volatile* const) 0x01d7007c) /* BCDDAY (BCD Day) */
#define BCDDATE ((uint32_t volatile* const) 0x01d70080) /* BCDDATE (BCD Date) */
#define BCDMON ((uint32_t volatile* const) 0x01d70084) /* BCDMON (BCD Month) */
#define BCDYEAR ((uint32_t volatile* const) 0x01d70088) /* BCDYEAR (BCD Year) */
#define TICINT ((uint32_t volatile* const) 0x01D7008C) /* TICINT (Tick time count) */

/****** CLOCK & POWER MANAGEMENT */
#define PLLCON ((uint32_t volatile* const) 0x01d80000) /* PLLCON (PLL Control) */
#define CLKCON ((uint32_t volatile* const) 0x01d80004) /* CLKCON (Clock Control) */
#define CLKSLOW ((uint32_t volatile* const) 0x01d80008) /* CLKSLOW (Slow clock Control) */
#define LOCKTIME ((uint32_t volatile* const) 0x01d8000c) /* LOCKTIME (PLL lock time Counter) */

/****** INTERRUPT CONTROLLER */
#define INTCON ((uint32_t volatile* const) 0x01e00000) /* INTCON (Interrupt Control) */
#define INTPND ((uint32_t volatile* const) 0x01e00004) /* INTPND (Interrupt Request Status) */
#define INTMOD ((uint32_t volatile* const) 0x01e00008) /* INTMOD (Interrupt Mode Control) */
#define INTMSK ((uint32_t volatile* const) 0x01e0000c) /* INTMSK (Interrupt Mask Control) */
#define I_PSLV ((uint32_t volatile* const) 0x01e00010) /* I_PSLV (IRQ Interrupt Previous Slave) */
#define I_PMST ((uint32_t volatile* const) 0x01e00014) /* I_PMST (IRQ Interrupt Priority Master) */
#define I_CSLV ((uint32_t volatile* const) 0x01e00018) /* I_CSLV (IRQ Interrupt Current Slave) */
#define I_CMST ((uint32_t volatile* const) 0x01e0001c) /* I_CMST (IRQ Interrupt Current Master) */
#define I_ISPR ((uint32_t volatile* const) 0x01e00020) /* I_ISPR (IRQ Interrupt Pending Status) */
#define I_ISPC ((uint32_t volatile* const) 0x01e00024) /* I_ISPC (IRQ Interrupt Pending Clear) */
#define F_ISPR ((uint32_t volatile* const) 0x01e00038) /* F_ISPR (FIQ Interrupt Pending) */
#define F_ISPC ((uint32_t volatile* const) 0x01e0003c) /* F_ISPC (FIQ Interrupt Pending Clear) */

/****** LCD CONTROLLER */
#define LCDCON1 ((uint32_t volatile* const) 0x01f00000) /* LCDCON1 (LCD Control 1) */
#define LCDCON2 ((uint32_t volatile* const) 0x01f00004) /* LCDCON2 (LCD Control 2) */
#define LCDCON3 ((uint32_t volatile* const) 0x01f00040) /* LCDCON3 (LCD Control 3) */
#define LCDSADDR1 ((uint32_t volatile* const) 0x01f00008) /* LCDSADDR1 (Frame Upper Buffer Start Address 1) */
#define LCDSADDR2 ((uint32_t volatile* const) 0x01f0000c) /* LCDSADDR2 (Frame Lower Buffer Start Address 2) */
#define LCDSADDR3 ((uint32_t volatile* const) 0x01f00010) /* LCDSADDR3 (Virtual Screen Address) */
#define REDLUT ((uint32_t volatile* const) 0x01f00014) /* REDLUT (RED Lookup Table) */
#define GREENLUT ((uint32_t volatile* const) 0x01f00018) /* GREENLUT (GREEN Lookup Table) */
#define BLUELUT ((uint32_t volatile* const) 0x01f0001c) /* BLUELUT (BLUE Lookup Table) */
#define DP1_2 ((uint32_t volatile* const) 0x01f00020) /* DP1_2 (Dithering Pattern duty 1/2) */
#define DP4_7 ((uint32_t volatile* const) 0x01f00024) /* DP4_7 (Dithering Pattern duty 4/7) */
#define DP3_5 ((uint32_t volatile* const) 0x01f00028) /* DP3_5 (Dithering Pattern duty 3/5) */
#define DP2_3 ((uint32_t volatile* const) 0x01f0002c) /* DP2_3 (Dithering Pattern duty 2/3) */
#define DP5_7 ((uint32_t volatile* const) 0x01f00030) /* DP5_7 (Dithering Pattern duty 5/7) */
#define DP3_4 ((uint32_t volatile* const) 0x01f00034) /* DP3_4 (Dithering Pattern duty 3/4) */
#define DP4_5 ((uint32_t volatile* const) 0x01f00038) /* DP4_5 (Dithering Pattern duty 4/5) */
#define DP6_7 ((uint32_t volatile* const) 0x01f0003c) /* DP6_7 (Dithering Pattern duty 6/7) */
#define DITHMODE ((uint32_t volatile* const) 0x01f00044) /* DITHMODE (Dithering Mode) */

/****** DMA */
#define ZDCON0 ((uint32_t volatile* const) 0x01e80000) /* ZDCON0 (ZDMA0 Control) */
#define ZDISRC0 ((uint32_t volatile* const) 0x01e80004) /* ZDISRC0 (ZDMA 0 Initial Source Address) */
#define ZDIDES0 ((uint32_t volatile* const) 0x01e80008) /* ZDIDES0 (ZDMA 0 Initial Destination Address) */
#define ZDICNT0 ((uint32_t volatile* const) 0x01e8000c) /* ZDICNT0 (ZDMA 0 Initial Transfer Count) */
#define ZDCSRC0 ((uint32_t volatile* const) 0x01e80010) /* ZDCSRC0 (ZDMA 0 Current Source Address) */
#define ZDCDES0 ((uint32_t volatile* const) 0x01e80014) /* ZDCDES0 (ZDMA 0 Current Destination Address) */
#define ZDCCNT0 ((uint32_t volatile* const) 0x01e80018) /* ZDCCNT0 (ZDMA 0 Current Transfer Count) */
#define ZDCON1 ((uint32_t volatile* const) 0x01e80020) /* ZDCON1 (ZDMA 1 Control) */
#define ZDISRC1 ((uint32_t volatile* const) 0x01e80024) /* ZDISRC1 (ZDMA 1 Initial Source Address) */
#define ZDIDES1 ((uint32_t volatile* const) 0x01e80028) /* ZDIDES1 (ZDMA 1 Initial Destination Address) */
#define ZDICNT1 ((uint32_t volatile* const) 0x01e8002c) /* ZDICNT1 (ZDMA 1 Initial Transfer Count) */
#define ZDCSRC1 ((uint32_t volatile* const) 0x01e80030) /* ZDCSRC1 (ZDMA 1 Current Source Address) */
#define ZDCDES1 ((uint32_t volatile* const) 0x01e80034) /* ZDCDES1 (ZDMA 1 Current Destination Address) */
#define ZDCCNT1 ((uint32_t volatile* const) 0x01e80038) /* ZDCCNT1 (ZDMA 1 Current Transfer Count) */
#define BDCON0 ((uint32_t volatile* const) 0x01f80000) /* BDCON0 (BDMA 0 Control) */
#define BDISRC0 ((uint32_t volatile* const) 0x01f80004) /* BDISRC0 (BDMA 0 Initial Source Address) */
#define BDIDES0 ((uint32_t volatile* const) 0x01f80008) /* BDIDES0 (BDMA 0 Initial Destination Address) */
#define BDICNT0 ((uint32_t volatile* const) 0x01f8000c) /* BDICNT0 (BDMA 0 Initial Transfer Count) */
#define BDCSRC0 ((uint32_t volatile* const) 0x01f80010) /* BDCSRC0 (BDMA 0 Current Source Address) */
#define BDCDES0 ((uint32_t volatile* const) 0x01f80014) /* BDCDES0 (BDMA 0 Current Destination Address) */
#define BDCCNT0 ((uint32_t volatile* const) 0x01f80018) /* BDCCNT0 (BDMA 0 Current Transfer Count) */
#define BDCON1 ((uint32_t volatile* const) 0x01f80020) /* BDCON1 (BDMA 1 Control) */
#define BDISRC1 ((uint32_t volatile* const) 0x01f80024) /* BDISRC1 (BDMA 1 Initial Source Address) */
#define BDIDES1 ((uint32_t volatile* const) 0x01f80028) /* BDIDES1 (BDMA 1 Initial Destination Address) */
#define BDICNT1 ((uint32_t volatile* const) 0x01f8002c) /* BDICNT1 (BDMA 1 Initial Transfer Count) */
#define BDCSRC1 ((uint32_t volatile* const) 0x01f80030) /* BDCSRC1 (BDMA 1 Current Source Address) */
#define BDCDES1 ((uint32_t volatile* const) 0x01f80034) /* BDCDES1 (BDMA 1 Current Destination Address) */
#define BDCCNT1 ((uint32_t volatile* const) 0x01f80038) /* BDCCNT1 (BDMA 1 Current Transfer Count) */

#endif
